@@ -1,32 +1,31 @@
 package VRML::Base;
 
 ############################## Copyright ##############################
-#								      #
-# This program is Copyright 1996,1998 by Hartmut Palm.		      #
-# This program is free software; you can redistribute it and/or	      #
-# modify it under the terms of the GNU General Public License	      #
+#                                                                     #
+# This program is Copyright 1996,1998 by Hartmut Palm.                #
+# This program is free software; you can redistribute it and/or       #
+# modify it under the terms of the GNU General Public License         #
 # as published by the Free Software Foundation; either version 2      #
-# of the License, or (at your option) any later version.	      #
-# 								      #
+# of the License, or (at your option) any later version.              #
+#                                                                     #
 # This program is distributed in the hope that it will be useful,     #
 # but WITHOUT ANY WARRANTY; without even the implied warranty of      #
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	      #
-# GNU General Public License for more details.			      #
-# 								      #
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       #
+# GNU General Public License for more details.                        #
+#                                                                     #
 # If you do not have a copy of the GNU General Public License write   #
 # to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     #
-# MA 02139, USA.						      #
-#								      #
+# MA 02139, USA.                                                      #
+#                                                                     #
 #######################################################################
 
 require 5.000;
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION $PI $PI_2);
 
-$VERSION = "1.03de";
-$::pi = 3.1415926;
-$::pi_2 = $::pi/2;
-$::pi_2 += 0; # prevent warnings
+$VERSION = "1.07de";
+$PI = 3.1415926;
+$PI_2 = $PI/2;
 
 sub new {
     my $class = shift;
@@ -59,7 +58,7 @@ sub browser {
     my $self = shift;
     return unless $_[0]; # @_ wouldn't work on PC
     ($self->{'BROWSER'}) = join("+",@_);
-    $self->VRML_put("# Set Browser to: '$self->{'browser'}'\n")
+    $self->_put("# Set Browser to: '$self->{'browser'}'\n")
       if $self->{'DEBUG'};
     return $self;
 }
@@ -69,28 +68,28 @@ sub browser {
 #
 #####################################################################
 
-sub VRML_init {
+sub _init {
     my $self = shift;
     my $vrml = shift;
     $self->{'VRML'} = [$vrml];
     return $self;
 }
 
-sub VRML_add {
+sub _add {
     my $self = shift;
     my $vrml = shift;
     ${$self->{'VRML'}}[$#{$self->{'VRML'}}] .= $vrml;
     return $self;
 }
 
-sub VRML_put {
+sub _put {
     my $self = shift;
     my $vrml = shift;
     push @{$self->{'VRML'}}, $vrml;
     return $self;
 }
 
-sub VRML_row {
+sub _row {
     my $self = shift;
     my ($row) = @_;
     my $vrml = $self->{'TAB'}.$row;
@@ -98,7 +97,7 @@ sub VRML_row {
     return $self;
 }
 
-sub VRML_swap {
+sub _swap {
     my $self = shift;
     my $element1 = pop @{$self->{'VRML'}};
     my $element2 = pop @{$self->{'VRML'}};
@@ -107,12 +106,12 @@ sub VRML_swap {
     return $self;
 }
 
-sub VRML_pos {
+sub _pos {
     my $self = shift;
     return $#{$self->{'VRML'}};
 }
 
-sub VRML_trim {
+sub _trim {
     my $self = shift;
     my ($index) = @_;
     $index = $#{$self->{'VRML'}} unless defined $index;
@@ -124,7 +123,7 @@ sub VRML_trim {
 sub debug {
     my $self = shift;
     $self->{'DEBUG'} = shift;
-    $self->VRML_put("# Set Debug Level to: $self->{'DEBUG'}\n");
+    $self->_put("# Set Debug Level to: $self->{'DEBUG'}\n");
     return $self;
 }
 
@@ -133,24 +132,32 @@ sub display_vars {
     my @keys = @_ ? @_ : sort keys %$self;
     my $key;
     foreach $key (@keys) {
-	unless (defined $self->{$key}) {
-	    print "# $key => undef\n";
-	    next;
-	}
-	print "# $key => $self->{$key}";
-	print " [".(join(', ',@{$self->{$key}}))."]" if defined
-	  ref($self->{$key}) && ref($self->{$key}) eq "ARRAY" &&
-	  $key ne "VRML";
-	print " [".(join(', ',sort keys %{$self->{$key}}))."]"
-	  if defined ref($self->{$key}) && ref($self->{$key}) eq "HASH";
-	print "\n";
+        unless (defined $self->{$key}) {
+            print "# $key => undef\n";
+            next;
+        }
+        print "# $key => $self->{$key}";
+        print " [".(join(', ',@{$self->{$key}}))."]" if defined
+          ref($self->{$key}) && ref($self->{$key}) eq "ARRAY" &&
+          $key ne "VRML";
+        print " [".(join(', ',sort keys %{$self->{$key}}))."]"
+          if defined ref($self->{$key}) && ref($self->{$key}) eq "HASH";
+        print "\n";
     }
     return $self;
 }
 
+sub string_to_array  {
+    my $self = shift;
+    my $pt = shift;
+    return @$pt if ref $pt eq 'ARRAY';
+    # remove leading/trailing spaces!
+    my $tmp = $pt; $tmp =~ s/^\s+//; $tmp =~ s/\s+$//;
+    return split(/\s+/,$tmp);
+}
 
 #--------------------------------------------------------------------
-#	Insert Comments
+#       Insert Comments
 #--------------------------------------------------------------------
 sub comment {
     my $self = shift;
@@ -161,28 +168,48 @@ sub comment {
 }
 
 #--------------------------------------------------------------------
-#	In-/Output VRML
+#       In-/Output VRML
 #--------------------------------------------------------------------
 sub insert {
     my $self = shift;
     my $string = shift;
+    $string =~ s/^\s+|\s+$//g;
+    $string = $self->{'TAB'}.$string;
+    $string =~ s/\n/\n$self->{'TAB'}/g;
     push @{$self->{'VRML'}}, $string."\n";
     return $self;
 }
 
 sub insert__DATA__ {
     my $self = shift;
+    $self->{'DATApos'} = tell(main::DATA) unless defined $self->{'DATApos'};
+    print "     self->{'DATApos'}=$self->{'DATApos'}\n" if $self->{'DEBUG'};
     push @{$self->{'VRML'}}, <main::DATA>,"\n";
+    seek(main::DATA,$self->{'DATApos'},0);
     return $self;
 }
 
 sub include {
     my $self = shift;
-    my $filename = shift;
-    return $self if !defined $filename || $filename eq "";
-    open(INCLUDE, "<$filename") || die;
-    push @{$self->{'VRML'}}, <INCLUDE>;
-    close(INCLUDE);
+    my @filename = @_;
+    return $self if !defined $filename[0] || $filename[0] eq "";
+    foreach (@filename) {
+        open(INCLUDE, "<$_") || die "Can't include \"$_\"\n$!\n";
+        push @{$self->{'VRML'}}, <INCLUDE>;
+        push @{$self->{'VRML'}}, "\n";
+        close(INCLUDE);
+    }
+    return $self;
+}
+
+sub format {
+    my $self = shift;
+    my $format = shift;
+    if (defined $format && $format eq "none") {
+        map { s/^[\t ]+//; s/\n[\t ]+/\n/g; s/\t+/ /g; s/ +/ /g; } @{$self->{'VRML'}};
+    } else {
+        map { s/    / /g; s/\t/  /g; } @{$self->{'VRML'}};
+    }
     return $self;
 }
 
@@ -194,12 +221,12 @@ sub print {
     print "Content-type: $self->{'Content-type'}\n\n"
       if $self->{'Content-type'} && $mime;
     if ($pipe) {
-	open(PIPE, "|$pipe") || die; select PIPE; $|=1;
-	for (@{$self->{'VRML'}}) { print; }
-	select STDOUT;
-	close(PIPE);
+        open(PIPE, "|$pipe") || die; select PIPE; $|=1;
+        for (@{$self->{'VRML'}}) { print; }
+        select STDOUT;
+        close(PIPE);
     } else {
-	for (@{$self->{'VRML'}}) { print; }
+        for (@{$self->{'VRML'}}) { print; }
     }
     return $self;
 }
@@ -209,15 +236,15 @@ sub save {
     my $filename = shift;
     my $pipe = shift;
     unless (defined $filename) {
-	($filename) = $0 =~ m/(.*)\./;
-	$filename .= ".wrl";
+        ($filename) = $0 =~ m/(.*)\./;
+        $filename .= ".wrl";
     }
     open(VRMLFILE, ">$filename") ||
       die "Can't create file: \"$filename\" ($!)\n";
     if ($pipe) {
-	print VRMLFILE "Can't pipe to \"$pipe\"\n";
-	close(VRMLFILE);
-	open(VRMLFILE, "| $pipe > $filename") || die;
+        print VRMLFILE "Can't pipe to \"$pipe\"\n";
+        close(VRMLFILE);
+        open(VRMLFILE, "| $pipe > $filename") || die;
     }
     for (@{$self->{'VRML'}}) { print VRMLFILE; }
     close(VRMLFILE);
@@ -240,23 +267,30 @@ sub escape {
 
 sub ascii {
     my $self = shift;
-    local $_ = shift;
-    s/[\204\344\365]/ae/g;
-    s/[\224\366]/oe/g;
-    s/[\201\374]/ue/g;
-    s/[\216\304]/Ae/g;
-    s/[\231\305\326]/Oe/g;
-    s/[\263\334]/Ue/g;
-    s/[\257\337]/sz/g;
-    s/[\000-\010\013-\037\177-\377]/_/g;
-    return $_;
+    local $_ = undef;
+    foreach (@_) {
+        s/[\204\344\365]/ae/g;
+        s/[\224\366]/oe/g;
+        s/[\201\374]/ue/g;
+        s/[\216\304]/Ae/g;
+        s/[\231\305\326]/Oe/g;
+        s/[\263\334]/Ue/g;
+        s/[\257\337]/sz/g;
+        s/[\000-\010\013-\037\177-\377]/_/g;
+    }
+    return (@_) if wantarray;
+    return $_[0];
 }
 
 sub utf8 {
     my $self = shift;
-    local $_ = shift;
-    s/[\000-\010\013-\037]/_/g;
-    return $_;
+    local $_ = undef;
+    #foreach (@_) {
+    #    s/[\201\374]/\x00\x75\x03\x08/g;
+    #    s/(.)/\x00$1/g;
+    #}
+    return (@_) if wantarray;
+    return $_[0];
 }
 
 sub xyz {
@@ -306,10 +340,9 @@ sub bboxSize {
 
 __END__
 
-
 =head1 NAME
 
-VRML::Base.pm - Basis-Methoden fr die VRML 1 und 2 Module
+VRML::Base.pm - Basis-Methoden fuer die VRML 1 und 2 Module
 
 =head1 SYNOPSIS
 
@@ -317,7 +350,7 @@ VRML::Base.pm - Basis-Methoden fr die VRML 1 und 2 Module
 
 =head1 DESCRIPTION
 
-Folgende Methoden stehen zur Verfgung.
+Folgende Methoden stehen zur Verfuegung.
 
 =over 4
 
@@ -325,7 +358,7 @@ Folgende Methoden stehen zur Verfgung.
 
 F<new>
 
-erzeugt ein neues VRML-Szenen-Objekt. Diese Methode mu vor der Verwendung
+erzeugt ein neues VRML-Szenen-Objekt. Diese Methode muss vor der Verwendung
 einer weiteren Methode aufgerufen werden.
 
 =item browser
@@ -338,13 +371,13 @@ F<browser("vrml", "html")>
 
 Die Implementierung der VRML-Spezifikationen in den Browsern und Plug-ins ist
 je nach Entwicklungsstand und Hersteller unterschiedlich. Um einige
-allgemeine Besonderheiten bercksichtigen zu knnen und Anzeigefehler zu
+allgemeine Besonderheiten beruecksichtigen zu koennen und Anzeigefehler zu
 verhindern, sollte deshalb der F<browser>-Methode der Name des VRML- und
 HTML-Browsers mitgegeben werden. I<Leere Parameter> oder das Weglassen der
 Methode bewirken die Verwendung des I<kleinsten vertretbaren
-Implementationsstandes> aller untersttzten VRML-Browser.
+Implementationsstandes> aller unterstuetzten VRML-Browser.
 
-Beispiele fr VRML-Browser:
+Beispiele fuer VRML-Browser:
 
     Cosmo Player 1.0
     Cosmo Player 2.0
@@ -356,24 +389,24 @@ Beispiele fr VRML-Browser:
     VRweb
     WorldView 2.0 Plugin
 
-Gebruchliche HTML-Browser:
+Gebraeuchliche HTML-Browser:
 
     Mozilla (Netscape)
     Mosaic
     MSIE (Microsoft Internet Explorer)
 
-Prinzipiell ist es auch mglich ber das API eines VRML-2.0-Browsers, den
+Prinzipiell ist es auch moeglich ueber das API eines VRML-2.0-Browsers, den
 Namen und die Version zu ermitteln. Dieses Verfahren besitzt jedoch einen
-entscheidenden Nachteil: Bevor ber das API die Informationen abgefragt
-werden knnen, mu die Szenenquelle bereits erstellt und erfolgreich geladen
-worden sein. Dann ist es aber fr syntaktische nderungen bereits zu spt.
+entscheidenden Nachteil: Bevor ueber das API die Informationen abgefragt
+werden koennen, muss die Szenenquelle bereits erstellt und erfolgreich geladen
+worden sein. Dann ist es aber fuer syntaktische Aenderungen bereits zu spaet.
 
-Ein Beispiel fr das unterschiedliche Verhalten der VRML-Browser ist die
-Interpretation eines escapten doppelten Anfhrungszeichens innerhalb einer
-Zeichenkette. Whrend einige Browser es, wie in der Spezifikation
-beschrieben, darstellen knnen, beenden andere Browser die Zeichenkette
+Ein Beispiel fuer das unterschiedliche Verhalten der VRML-Browser ist die
+Interpretation eines escapten doppelten Anfuehrungszeichens innerhalb einer
+Zeichenkette. Waehrend einige Browser es, wie in der Spezifikation
+beschrieben, darstellen koennen, beenden andere Browser die Zeichenkette
 vorzeitig und erzeugen somit weitere Syntaxfehler. Ein weiteres Problem ist
-die untersttzte Sprache im Script-Knoten. Hier mu bei einigen Browsern
+die unterstuetzte Sprache im Script-Knoten. Hier muss bei einigen Browsern
 'vrmlscript' angegeben werden.
 
 Beispiel:
@@ -386,9 +419,9 @@ F<comment('string')>
 
  string MFString []
 
-fgt an der aktuellen Szenenposition einen Kommentar ein. Jeder Zeichenkette
+fuegt an der aktuellen Szenenposition einen Kommentar ein. Jeder Zeichenkette
 aus dem Parameter I<string> wird ein Doppelkreuz vorangestellt und ein
-Zeilenvorschub angefgt.
+Zeilenvorschub angefuegt.
 
 =item insert
 
@@ -396,8 +429,8 @@ F<insert('string')>
 
  string SFString ""
 
-fgt vorhandenen VRML-Code in die Szene ein. Dieser kann als skalare Variable
-oder als konstante Zeichenkette dem Parameter I<string> bergeben werden.
+fuegt vorhandenen VRML-Code in die Szene ein. Dieser kann als skalare Variable
+oder als konstante Zeichenkette dem Parameter I<string> uebergeben werden.
 
 Beispiel:
 
@@ -407,9 +440,9 @@ Beispiel:
     ->end
     ->print;
 
-Befinden sich im vorhandenen VRML-Code doppelte Anfhrungszeichen, so sollte
-die Perl-Funktion qq verwendet werden, um den Code unverndert bernehmen zu
-knnen. Alternativ dazu besteht die Mglichkeit, die Anfhrungszeichen durch
+Befinden sich im vorhandenen VRML-Code doppelte Anfuehrungszeichen, so sollte
+die Perl-Funktion qq verwendet werden, um den Code unveraendert uebernehmen zu
+koennen. Alternativ dazu besteht die Moeglichkeit, die Anfuehrungszeichen durch
 einen Backslash zu maskieren (\" ).
 
     $vrml
@@ -419,8 +452,8 @@ einen Backslash zu maskieren (\" ).
     ->print;
 
 
-Der Szenenaufbau kann schnell unbersichtlich werden, wenn der VRML-Code
-einige Zeilen berschreitet. Fr das Einfgen grerer Programmteile ist die
+Der Szenenaufbau kann schnell unuebersichtlich werden, wenn der VRML-Code
+einige Zeilen ueberschreitet. Fuer das Einfuegen groesserer Programmteile ist die
 Methode C<insert__DATA__> besser geeignet.
 
 
@@ -430,8 +463,8 @@ F<insert__DATA__()>
 
 macht sich die Perl-Syntax zu nutze, in der alle folgenden Zeilen nach der
 Zeichenkette __DATA__ als Daten behandelt werden. Diese liest die Methode
-F<insert__DATA__> ein und fgt sie an der betreffenden Stelle in die Szene
-ein. Beachte die fhrenden und abschlieenden ZWEI Unterstriche.
+F<insert__DATA__> ein und fuegt sie an der betreffenden Stelle in die Szene
+ein. Beachte die fuehrenden und abschliessenden ZWEI Unterstriche.
 
 Beispiel:
 
@@ -446,14 +479,14 @@ Beispiel:
     Shape {
       geometry Sphere {}
       appearance Appearance {
-	material Material {
-	  diffuseColor 0 0.5 0
-	}
+        material Material {
+          diffuseColor 0 0.5 0
+        }
       }
     }
 
 B<Hinweis:> Der __DATA__-Abschnitt in Perl-Skripten wird derzeit nicht von
-C<modperl> auf dem Apache-Server untersttzt. D.h. F<insert__DATA__>
+C<modperl> auf dem Apache-Server unterstuetzt. D.h. F<insert__DATA__>
 funktioniert dort nicht wie erwartet.
 
 =item include
@@ -462,7 +495,7 @@ F<include('files')>
 
  files MFString []
 
-fgt vorhandene VRML-Dateien in die aktuelle Szene ein. Der Parameter
+fuegt vorhandene VRML-Dateien in die aktuelle Szene ein. Der Parameter
 I<files> kann eine Liste von Dateinamen enthalten, die der Reihenfolge nach
 eingebunden werden.
 
@@ -477,21 +510,21 @@ F<print('mime', 'pipe')>
  mime SFBool   0
  pipe SFString ""
 
-bergibt den Inhalt des Szenenobjekts an STDOUT. Das bedeutet im Normalfall,
-da die VRML-Quelle auf dem Bildschirm erscheint. Wird das Skript von einem
-WWW-Server ber CGI gestartet, so bentigt der Client (Browser) einen
-MIME-Typ, um die korrekte Wiedergabeart zu ermitteln. Der MIME-Typ mu im
-Header vor der eigentlichen Szene gesendet werden. ber den Parameter I<mime>
+uebergibt den Inhalt des Szenenobjekts an STDOUT. Das bedeutet im Normalfall,
+dass die VRML-Quelle auf dem Bildschirm erscheint. Wird das Skript von einem
+WWW-Server ueber CGI gestartet, so benoetigt der Client (Browser) einen
+MIME-Typ, um die korrekte Wiedergabeart zu ermitteln. Der MIME-Typ muss im
+Header vor der eigentlichen Szene gesendet werden. Ueber den Parameter I<mime>
 kann diese Option aktiviert werden.
 
-Um die bertragungs- bzw. Ladezeiten virtueller Welten zu verkrzen, besteht
-die Mglichkeit, VRML-Quellen zu komprimieren. Zu diesem Zweck wird ein
-Programm bentigt, welches das GNU-ZIP-Verfahren realisiert. ber den
-Parameter pipe mssen der Pfad, Name und die Programmparameter der
-ausfhrbaren Datei spezifiziert werden. Befindet sich die Datei im aktuellen
-Pfad, gengt nur der Name und die Parameter (meistens C<gzip -f>). Der
+Um die Uebertragungs- bzw. Ladezeiten virtueller Welten zu verkuerzen, besteht
+die Moeglichkeit, VRML-Quellen zu komprimieren. Zu diesem Zweck wird ein
+Programm benoetigt, welches das GNU-ZIP-Verfahren realisiert. Ueber den
+Parameter pipe muessen der Pfad, Name und die Programmparameter der
+ausfuehrbaren Datei spezifiziert werden. Befindet sich die Datei im aktuellen
+Pfad, genuegt nur der Name und die Parameter (meistens C<gzip -f>). Der
 Parameter I<pipe> ist jedoch nicht nur auf das Komprimieren der VRML-Skripte
-beschrnkt. Prinzipiell kann hier jeder Filter angewendet werden.
+beschraenkt. Prinzipiell kann hier jeder Filter angewendet werden.
 
 Beispiel 1:
 
@@ -508,7 +541,7 @@ Beispiel 3 (MS-DOS gzip.exe):
     $vrml->print(1,"c:\\Perl\\bin\\gzip.exe -f");
 
 
-oder fr alle Plattformen, wenn sich das Programm C<gzip> im Suchpfad
+oder fuer alle Plattformen, wenn sich das Programm C<gzip> im Suchpfad
 befindet:
 
     $vrml->print(1,"gzip -f");
@@ -523,12 +556,12 @@ F<save('filename', 'pipe')>
 
 speichert den Inhalt des Szenenobjekts in einer Datei. Wird kein Dateiname
 angegeben, so wird die Erweiterung des gerade abgearbeiteten Skripts (z. B.
-.pl) gegen die Erweiterung '.wrl' ausgetauscht. Um bei groen Welten
-Speicherplatz zu sparen, besteht auch hier die Mglichkeit, die VRML-Datei zu
-komprimieren. Zu diesem Zweck wird ein Programm bentigt, das ein beliebiges
-Pack-Verfahren realisiert. Besonders gut eignet sich dafr das
-GNU-ZIP-Verfahren, da es vom VRML-Browser selbst entpackt werden kann. ber
-den Parameter I<pipe> mu der Name und Pfad der ausfhrbaren Datei
+.pl) gegen die Erweiterung '.wrl' ausgetauscht. Um bei grossen Welten
+Speicherplatz zu sparen, besteht auch hier die Moeglichkeit, die VRML-Datei zu
+komprimieren. Zu diesem Zweck wird ein Programm benoetigt, das ein beliebiges
+Pack-Verfahren realisiert. Besonders gut eignet sich dafuer das
+GNU-ZIP-Verfahren, da es vom VRML-Browser selbst entpackt werden kann. Ueber
+den Parameter I<pipe> muss der Name und Pfad der ausfuehrbaren Datei
 spezifiziert werden. Die Funktionsweise von I<pipe> ist analog der in der
 Methodenbeschreibung von C<print>.
 
@@ -551,9 +584,9 @@ Beispiel 3:
 
 F<as_string()>
 
-gibt die komplette VRML-Quelle als Zeichenkette zurck. Sie wird jedoch nur
-in seltenen Fllen bentigt und ist die einzige Methode, welche nicht eine
-Referenz auf das Szenenobjekt zurckliefert. Fr die Ausgabe oder Speicherung
+gibt die komplette VRML-Quelle als Zeichenkette zurueck. Sie wird jedoch nur
+in seltenen Faellen benoetigt und ist die einzige Methode, welche nicht eine
+Referenz auf das Szenenobjekt zurueckliefert. Fuer die Ausgabe oder Speicherung
 einer VRML-Quelle sollten im allgemeinen die Methoden C<print> oder C<save>
 benutzt werden. Diese Methoden sind wesentlich effizienter und schonen die
 Ressourcen des Rechners.
@@ -565,12 +598,21 @@ Ressourcen des Rechners.
     $scene = $vrml->as_string;
 
 
+=item ascii
+
+F<ascii('string')>
+
+transformiert alle Umlaute in ae, oe, ue und sz. Weiterhin enfernt die Methode
+alle Steuerzeichen.
+
+    $vrml->text($vrml->ascii("umlaute"), ' yellow')
+
 
 =back
 
 Folgende Methoden sollten nicht verwendet werden. Sie sind normalerweise
 nicht notwendig. Besteht dennoch Bedarf, so sende mir bitte eine E-Mail und
-ich werde sie in den nchsten Versionen beschreiben.
+ich werde sie in den naechsten Versionen beschreiben.
 
 =over 4
 
