@@ -1,38 +1,42 @@
 package VRML::VRML1;
 
 ############################## Copyright ##############################
-#								      #
-# This program is Copyright 1996,1998 by Hartmut Palm.		      #
-# This program is free software; you can redistribute it and/or	      #
-# modify it under the terms of the GNU General Public License	      #
+#                                                                     #
+# This program is Copyright 1996,1998 by Hartmut Palm.                #
+# This program is free software; you can redistribute it and/or       #
+# modify it under the terms of the GNU General Public License         #
 # as published by the Free Software Foundation; either version 2      #
-# of the License, or (at your option) any later version.	      #
-# 								      #
+# of the License, or (at your option) any later version.              #
+#                                                                     #
 # This program is distributed in the hope that it will be useful,     #
 # but WITHOUT ANY WARRANTY; without even the implied warranty of      #
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	      #
-# GNU General Public License for more details.			      #
-# 								      #
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       #
+# GNU General Public License for more details.                        #
+#                                                                     #
 # If you do not have a copy of the GNU General Public License write   #
 # to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     #
-# MA 02139, USA.						      #
-#								      #
+# MA 02139, USA.                                                      #
+#                                                                     #
 #######################################################################
 
 require 5.000;
 require VRML::VRML1::Standard;
 use strict;
 use VRML::Color;
+use VRML::Base;
 use vars qw(@ISA $AUTOLOAD $VERSION %supported);
 @ISA = qw(VRML::VRML1::Standard);
 
-$VERSION="1.03de";
+$VERSION = "1.10de";
 %supported = ('quote' => "Live3D|WorldView|Cosmo Player",
- 'L3D_ext' => "Live3D|Cosmo Player",	# not WorldView
+ 'L3D_ext' => "Live3D|Cosmo Player",    # not WorldView
  'gzip'   => "Live3D|WorldView|Cosmo Player|libcosmoplayer|VRweb|GLview",
  'target' => "Live3D|WorldView|Cosmo Player|libcosmoplayer|MSVRML2OCX"
 );
 
+my $PI = 3.1415926;
+my $PI_2 = $PI / 2;
+
 #--------------------------------------------------------------------
 
 sub new {
@@ -78,13 +82,13 @@ sub end {
 
 sub anchor_begin {
     my $self = shift;
-    return $self->VRML_put(qq{# CALL: ->anchor_begin("Url","description","target=parameter");\n}) unless @_;
+    return $self->_put(qq{# CALL: ->anchor_begin("Url","description","target=parameter");\n}) unless @_;
     my ($url, $description, $parameter) = @_;
     my $target = undef;
     my $quote = $self->{'BROWSER'} =~ /$supported{'quote'}/i ? '\\"' : "'";
     $description =~ s/"/$quote/g if defined $description;
     if (defined $parameter && $self->{'BROWSER'} =~ /$supported{'target'}/i) {
-	($target = $1) =~ s/"/$quote/g if ($parameter =~ /target\s*=(.+)/i);
+        ($target = $1) =~ s/"/$quote/g if ($parameter =~ /target\s*=(.+)/i);
     }
     $self->WWWAnchor($url, $description, $target);
     return $self;
@@ -98,8 +102,8 @@ sub anchor_end {
 
 sub collision_begin {
     my $self = shift;
-    $self->VRML_row("CollideStyle { collide TRUE }\n") if $self->{'BROWSER'} =~ /$supported{'L3D_ext'}/i;
-    $self->VRML_row("DEF CollisionDetection Info { string \"TRUE\" }\n");
+    $self->_row("CollideStyle { collide TRUE }\n") if $self->{'BROWSER'} =~ /$supported{'L3D_ext'}/i;
+    $self->_row("DEF CollisionDetection Info { string \"TRUE\" }\n");
     return $self;
 }
 
@@ -121,7 +125,7 @@ sub group_end {
 
 sub lod_begin {
     my $self = shift;
-    return $self->VRML_put(qq{# CALL: ->lod_begin(range,"center");\n}) unless @_;
+    return $self->_put(qq{# CALL: ->lod_begin(range,"center");\n}) unless @_;
     my ($range, $center) = @_;
     $self->LOD($range,$center);
     return $self;
@@ -170,8 +174,8 @@ sub backgroundcolor {
     my ($skyColorString) = @_;
     my $skyColor;
     if (defined $skyColorString) {
-	$skyColor = rgb_color($skyColorString);
-	$self->def("BackgroundColor")->Info($skyColor)->VRML_trim;
+        $skyColor = rgb_color($skyColorString);
+        $self->def("BackgroundColor")->Info($skyColor)->_trim;
     }
     return $self;
 }
@@ -180,28 +184,28 @@ sub backgroundimage {
     my $self = shift;
     my $bgimage = shift;
     if (defined $bgimage) {
-	$self->def("BackgroundImage")->Info($bgimage)->VRML_trim;
+        $self->def("BackgroundImage")->Info($bgimage)->_trim;
     }
     return $self;
 }
 
 sub title {
     my $self = shift;
-    return $self->VRML_put(qq{# CALL: ->title("string");\n}) unless @_;
+    return $self->_put(qq{# CALL: ->title("string");\n}) unless @_;
     my $title = shift;
-    $self->VRML_row("DEF Title Info { string \"$title\" }\n");
+    $self->_row("DEF Title Info { string \"$title\" }\n");
     return $self;
 }
 
 sub info {
     my $self = shift;
     my $string = shift;
-    return $self->VRML_put(qq{# CALL: ->info("string");\n}) unless @_;
+    return $self->_put(qq{# CALL: ->info("string");\n}) unless @_;
     my $quote = $self->{'BROWSER'} =~ /$supported{'quote'}/i ? '\\"' : "'";
     if (defined $string) {
-	$string = join(',',@$string) if ref($string) eq "ARRAY";
-	$string =~ s/"/$quote/g;
-	$self->VRML_row("Info { string \"$string\" }\n");
+        $string = join(',',@$string) if ref($string) eq "ARRAY";
+        $string =~ s/"/$quote/g;
+        $self->_row("Info { string \"$string\" }\n");
     }
     return $self;
 }
@@ -211,11 +215,11 @@ sub worldinfo {
     my $title = shift;
     my $string = shift;
     my $quote = $self->{'BROWSER'} =~ /$supported{'quote'}/i ? '\\"' : "'";
-    $self->VRML_row("DEF Title Info { string \"$title\" }\n") if defined $title;
+    $self->_row("DEF Title Info { string \"$title\" }\n") if defined $title;
     if (defined $string) {
-	$string = join(',',@$string) if ref($string) eq "ARRAY";
-	$string =~ s/"/$quote/g;
-	$self->VRML_row("Info { string \"$string\" }\n");
+        $string = join(',',@$string) if ref($string) eq "ARRAY";
+        $string =~ s/"/$quote/g;
+        $self->_row("Info { string \"$string\" }\n");
     }
     return $self;
 }
@@ -224,10 +228,10 @@ sub navigationinfo {
     my $self = shift;
     my ($type, $speed, $headlight) = @_;
     $type = $$type[0] if ref($type) eq "ARRAY";
-    $self->VRML_row("DEF Viewer Info { string \"$type\" }\n");
-    $self->VRML_row("DEF ViewerSpeed Info { string \"$speed\" }\n");
+    $self->_row("DEF Viewer Info { string \"$type\" }\n");
+    $self->_row("DEF ViewerSpeed Info { string \"$speed\" }\n");
     $headlight = defined $headlight && !$headlight ? "FALSE" : "TRUE";
-    $self->VRML_row("DEF Headlight Info { string \"$headlight\" }\n");
+    $self->_row("DEF Headlight Info { string \"$headlight\" }\n");
     return $self;
 }
 #--------------------------------------------------------------------
@@ -237,7 +241,7 @@ sub viewpoint_begin {
     my ($whichChild) = @_;
     $whichChild = (defined $whichChild && $whichChild > 0) ? $whichChild-1 : 0;
     my $vrml = $self->{'TAB'}."DEF Cameras Switch {\n";
-    $vrml .= $self->{'TAB'}."	whichChild $whichChild\n";
+    $vrml .= $self->{'TAB'}."   whichChild $whichChild\n";
     $self->{'TAB_VIEW'} = $self->{'TAB'}."\t";
     $self->{'viewpoint_begin'} = $#{$self->{'VRML'}}+1 unless defined $self->{'viewpoint_begin'};
     push @{$self->{'viewpoint'}}, $vrml;
@@ -258,22 +262,22 @@ sub viewpoint_auto_set {
     my $factor = shift;
     $factor = 1 unless defined $factor;
     if (defined $self->{'viewpoint_set'}) {
-	my $x = ($self->{'Xmax'}+$self->{'Xmin'})/2;
-	my $y = ($self->{'Ymax'}+$self->{'Ymin'})/2;
-	my $z = ($self->{'Zmax'}+$self->{'Zmin'})/2;
-	my $dx = abs($self->{'Xmax'}-$x); # todo: calculate angle
-	my $dy = abs($self->{'Ymax'}-$y);
-	my $dz = abs($self->{'Zmax'}-$z);
-	my $dist = 0;
-	$dist = $dx if $dx > $dist;
-	$dist = $dy if $dy > $dist;
-	$dist = $dz if $dz > $dist;
-	my $offset = $#{$self->{'viewpoint'}}+1;
-	$self->viewpoint_set("$x $y $z",$dist*$factor,60);
-	@_ = splice(@{$self->{'viewpoint'}}, $offset);
-	splice(@{$self->{'viewpoint'}}, $self->{'viewpoint_set'}, $#_+1, @_);
+        my $x = ($self->{'Xmax'}+$self->{'Xmin'})/2;
+        my $y = ($self->{'Ymax'}+$self->{'Ymin'})/2;
+        my $z = ($self->{'Zmax'}+$self->{'Zmin'})/2;
+        my $dx = abs($self->{'Xmax'}-$x); # todo: calculate angle
+        my $dy = abs($self->{'Ymax'}-$y);
+        my $dz = abs($self->{'Zmax'}-$z);
+        my $dist = 0;
+        $dist = $dx if $dx > $dist;
+        $dist = $dy if $dy > $dist;
+        $dist = $dz if $dz > $dist;
+        my $offset = $#{$self->{'viewpoint'}}+1;
+        $self->viewpoint_set("$x $y $z",$dist*$factor,60);
+        @_ = splice(@{$self->{'viewpoint'}}, $offset);
+        splice(@{$self->{'viewpoint'}}, $self->{'viewpoint_set'}, $#_+1, @_);
     } else {
-	$self->viewpoint_set(@_);
+        $self->viewpoint_set(@_);
     }
     return $self;
 }
@@ -282,8 +286,8 @@ sub viewpoint_set {
     my $self = shift;
     my ($center, $distance, $heightAngle) = @_;
     $self->{'viewpoint_set'} = $#{$self->{'viewpoint'}}+1 unless defined $self->{'viewpoint_set'};
-    my ($x, $y, $z) = split(/\s+/,$center) if defined $center;
-    my ($dx, $dy, $dz) = defined $distance ? split(/\s+/,$distance) : (0,0,0);
+    my ($x, $y, $z) = $self->string_to_array($center) if defined $center;
+    my ($dx, $dy, $dz) = defined $distance ? $self->string_to_array($distance) : (0,0,0);
     $x = 0 unless defined $x;
     $y = 0 unless defined $y;
     $z = 0 unless defined $z;
@@ -303,23 +307,23 @@ sub viewpoint {
     my $self = shift;
     my ($name, $position, $orientation, $heightAngle) = @_;
     if (defined $orientation) {
-	if ($orientation !~ /\s/) {
-	    my %val = ("FRONT" => "0 0 1 0", "BACK" => "0 1 0 3.14",
-	    	"RIGHT" => "0 1 0  1.57", "LEFT" => "0 1 0 -1.57",
-		"TOP" => "1 0 0 -1.57",	"BOTTOM" => "1 0 0 1.57");
-	    my $string = uc($orientation);
-	    undef $orientation;
-	    $orientation = $val{$string};
+        if ($orientation !~ /\s/) {
+            my %val = ("FRONT" => "0 0 1 0", "BACK" => "0 1 0 3.14",
+                "RIGHT" => "0 1 0  1.57", "LEFT" => "0 1 0 -1.57",
+                "TOP" => "1 0 0 -1.57", "BOTTOM" => "1 0 0 1.57");
+            my $string = uc($orientation);
+            undef $orientation;
+            $orientation = $val{$string};
             $orientation .= " # $string" if $orientation && $self->{'DEBUG'};
-	} else {
-	    my ($x,$y,$z,$angle) = ref($orientation) ? @$orientation : split(/\s+/,$orientation);
-	    if (defined $angle) {
-		$angle *= $::pi/180 if $self->{'CONVERT'};
-		$orientation = "$x $y $z $angle";
-	    }
-	}
+        } else {
+            my ($x,$y,$z,$angle) = $self->string_to_array($orientation);
+            if (defined $angle) {
+                $angle *= $PI/180 if $self->{'CONVERT'};
+                $orientation = "$x $y $z $angle";
+            }
+        }
     }
-    $heightAngle *= $::pi/180 if defined $heightAngle && $self->{'CONVERT'};
+    $heightAngle *= $PI/180 if defined $heightAngle && $self->{'CONVERT'};
     $self->{'TAB_VIEW'} = $self->{'TAB'} unless $self->{'TAB_VIEW'};
     $name =~ s/^#//;
     $name =~ s/[\x00-\x20\x22\x23\x27\x2b-\x2e\x30-\x39\x5b-\x5d\x7b\x7d\x7f]/_/g;
@@ -327,8 +331,8 @@ sub viewpoint {
     $self->PerspectiveCamera($position, $orientation, $heightAngle);
     push @{$self->{'viewpoint'}}, pop @{$self->{'VRML'}};
     unless (defined $self->{'viewpoint_begin'}) {
-	splice(@{$self->{'VRML'}}, @{$self->{'VRML'}}, 0, @{$self->{'viewpoint'}});
-	$self->{'viewpoint'} = [];
+        splice(@{$self->{'VRML'}}, @{$self->{'VRML'}}, 0, @{$self->{'viewpoint'}});
+        $self->{'viewpoint'} = [];
     }
     return $self;
 }
@@ -348,10 +352,10 @@ sub directionallight {
 
 sub line {
     my $self = shift;
-    return $self->VRML_put(qq{# CALL: ->line("fromXYZ","toXYZ",radius,"appearance","[x][y][z]");\n}) unless @_;
+    return $self->_put(qq{# CALL: ->line("fromXYZ","toXYZ",radius,"appearance","[x][y][z]");\n}) unless @_;
     my ($from,$to,$radius,$appearance,$order) = @_;
-    my ($x1,$y1,$z1) = ref($from) ? @$from : split(/\s+/,$from);
-    my ($x2,$y2,$z2) = ref($to) ? @$to : split(/\s+/,$to);
+    my ($x1,$y1,$z1) = $self->string_to_array($from);
+    my ($x2,$y2,$z2) = $self->string_to_array($to);
     my ($t, $r, $length);
 
     $x1 = 0 unless $x1;
@@ -368,57 +372,57 @@ sub line {
     $self->comment('line("'.join('", "',@_).'")') if $self->{'DEBUG'};
     $self->Separator;
     if ($appearance) {
-	if (defined $radius && $radius==0 && ($self->{'BROWSER'} =~ /Cosmo Player/)) {
-	     $self->appearance("$appearance, 0 0 0");
-	} else {
-	     $self->appearance($appearance);
-	}
+        if (defined $radius && $radius==0 && ($self->{'BROWSER'} =~ /Cosmo Player/)) {
+             $self->appearance("$appearance, 0 0 0");
+        } else {
+             $self->appearance($appearance);
+        }
     }
     if (defined $radius && $radius>0) {
-	if ($dx && $order =~ /x/) {
-	    $self->Separator("line_x");
-	    $t = ($x1-($dx/2))." $y1 $z1" if $order =~ /^x$/i;
-	    $t = ($x1-($dx/2))." $y1 $z1" if $order =~ /^x../i;
-	    $t = ($x1-($dx/2))." $y2 $z1" if $order =~ /yxz/i;
-	    $t = ($x1-($dx/2))." $y1 $z2" if $order =~ /zxy/i;
-	    $t = ($x1-($dx/2))." $y2 $z2" if $order =~ /..x$/i;
-	    $self->Transform($t,"0 0 1 $::pi_2");
-	    $self->Cylinder($radius,abs($dx));
-	    $self->End();
-	}
-	if ($dy && $order =~ /y/) {
-	    $self->Separator("line_y");
-	    $t = "$x1 ".($y1-($dy/2))." $z1" if $order =~ /^y$/i;
-	    $t = "$x1 ".($y1-($dy/2))." $z1" if $order =~ /^y../i;
-	    $t = "$x2 ".($y1-($dy/2))." $z1" if $order =~ /xyz/i;
-	    $t = "$x1 ".($y1-($dy/2))." $z2" if $order =~ /zyx/i;
-	    $t = "$x2 ".($y1-($dy/2))." $z2" if $order =~ /..y$/i;
-	    $self->Transform($t);
-	    $self->Cylinder($radius,abs($dy));
-	    $self->End();
-	}
-	if ($dz && $order =~ /z/) {
-	    $self->Separator("line_z");
-	    $t = "$x1 $y1 ".($z1-($dz/2)) if $order =~ /^z$/i;
-	    $t = "$x1 $y1 ".($z1-($dz/2)) if $order =~ /^z../i;
-	    $t = "$x1 $y2 ".($z1-($dz/2)) if $order =~ /yzx/i;
-	    $t = "$x2 $y1 ".($z1-($dz/2)) if $order =~ /xzy/i;
-	    $t = "$x2 $y2 ".($z1-($dz/2)) if $order =~ /..z$/i;
-	    $self->Transform($t,"1 0 0 $::pi_2");
-	    $self->Cylinder($radius,abs($dz));
-	    $self->End();
-	}
-	unless ($order) {
-	    $length = sqrt($dx*$dx + $dy*$dy + $dz*$dz);
-	    $t = ($x1-($dx/2))." ".($y1-($dy/2))." ".($z1-($dz/2));
-	    $r = "$dx ".($dy+$length)." $dz $::pi";
-	    $self->Transform($t,$r);
-	    $self->Cylinder($radius,$length);
-	}
+        if ($dx && $order =~ /x/) {
+            $self->Separator("line_x");
+            $t = ($x1-($dx/2))." $y1 $z1" if $order =~ /^x$/i;
+            $t = ($x1-($dx/2))." $y1 $z1" if $order =~ /^x../i;
+            $t = ($x1-($dx/2))." $y2 $z1" if $order =~ /yxz/i;
+            $t = ($x1-($dx/2))." $y1 $z2" if $order =~ /zxy/i;
+            $t = ($x1-($dx/2))." $y2 $z2" if $order =~ /..x$/i;
+            $self->Transform($t,"0 0 1 ".$PI_2);
+            $self->Cylinder($radius,abs($dx));
+            $self->End();
+        }
+        if ($dy && $order =~ /y/) {
+            $self->Separator("line_y");
+            $t = "$x1 ".($y1-($dy/2))." $z1" if $order =~ /^y$/i;
+            $t = "$x1 ".($y1-($dy/2))." $z1" if $order =~ /^y../i;
+            $t = "$x2 ".($y1-($dy/2))." $z1" if $order =~ /xyz/i;
+            $t = "$x1 ".($y1-($dy/2))." $z2" if $order =~ /zyx/i;
+            $t = "$x2 ".($y1-($dy/2))." $z2" if $order =~ /..y$/i;
+            $self->Transform($t);
+            $self->Cylinder($radius,abs($dy));
+            $self->End();
+        }
+        if ($dz && $order =~ /z/) {
+            $self->Separator("line_z");
+            $t = "$x1 $y1 ".($z1-($dz/2)) if $order =~ /^z$/i;
+            $t = "$x1 $y1 ".($z1-($dz/2)) if $order =~ /^z../i;
+            $t = "$x1 $y2 ".($z1-($dz/2)) if $order =~ /yzx/i;
+            $t = "$x2 $y1 ".($z1-($dz/2)) if $order =~ /xzy/i;
+            $t = "$x2 $y2 ".($z1-($dz/2)) if $order =~ /..z$/i;
+            $self->Transform($t,"1 0 0 ".$PI_2);
+            $self->Cylinder($radius,abs($dz));
+            $self->End();
+        }
+        unless ($order) {
+            $length = sqrt($dx*$dx + $dy*$dy + $dz*$dz);
+            $t = ($x1-($dx/2))." ".($y1-($dy/2))." ".($z1-($dz/2));
+            $r = "$dx ".($dy+$length)." $dz ".$PI;
+            $self->Transform($t,$r);
+            $self->Cylinder($radius,$length);
+        }
     } else {
-	$self->MaterialBinding("PER_FACE");
-	$self->Coordinate3($from,$to);
-	$self->IndexedLineSet(["0, 1"]);
+        $self->MaterialBinding("PER_FACE");
+        $self->Coordinate3($from,$to);
+        $self->IndexedLineSet(["0, 1"]);
     }
     $self->End("line");
     return $self;
@@ -429,7 +433,7 @@ sub line {
 sub box {
     my $self = shift;
     my ($dimension, $appearance) = @_;
-    my ($width,$height,$depth) = ref($dimension) ? @$dimension : split(/\s+/,$dimension);
+    my ($width,$height,$depth) = $self->string_to_array($dimension);
     $self->Group->appearance($appearance) if $appearance;
     $self->Cube($width,$height,$depth);
     $self->End if $appearance;
@@ -439,7 +443,7 @@ sub box {
 sub cone {
     my $self = shift;
     my ($dimension, $appearance) = @_;
-    my ($radius, $height) = ref($dimension) ? @$dimension : split(/\s+/,$dimension);
+    my ($radius, $height) = $self->string_to_array($dimension);
     $self->Group->appearance($appearance) if $appearance;
     $self->Cone($radius, $height);
     $self->End if $appearance;
@@ -449,7 +453,7 @@ sub cone {
 sub cube {
     my $self = shift;
     my ($dimension, $appearance) = @_;
-    my ($width,$height,$depth) = ref($dimension) ? @$dimension : split(/\s+/,$dimension);
+    my ($width,$height,$depth) = $self->string_to_array($dimension);
     $height=$width unless defined $height;
     $depth=$width unless defined $depth;
     $self->Group->appearance($appearance) if $appearance;
@@ -461,16 +465,16 @@ sub cube {
 sub cylinder {
     my $self = shift;
     my ($dimension, $appearance, $top, $side, $bottom) = @_;
-    my ($radius, $height) = ref($dimension) ? @$dimension : split(/\s+/,$dimension);
+    my ($radius, $height) = $self->string_to_array($dimension);
     my @parts;
     $self->Group->appearance($appearance) if $appearance;
     if (defined $top || defined $side || defined $bottom) {
-	$top = 1 unless defined $top;
-	$side = 1 unless defined $side;
-	$bottom = 1 unless defined $bottom;
-	push @parts, "TOP" if $top;
-	push @parts, "SIDES" if $side;
-	push @parts, "BOTTOM" if $bottom;
+        $top = 1 unless defined $top;
+        $side = 1 unless defined $side;
+        $bottom = 1 unless defined $bottom;
+        push @parts, "TOP" if $top;
+        push @parts, "SIDES" if $side;
+        push @parts, "BOTTOM" if $bottom;
     }
     $self->Cylinder($radius, $height, @parts);
     $self->End if $appearance;
@@ -480,17 +484,17 @@ sub cylinder {
 sub pyramid {
     my $self = shift;
     my ($dimension, $appearance) = @_;
-    my ($width,$height,$depth) = ref($dimension) ? @$dimension : split(/\s+/,$dimension);
+    my ($width,$height,$depth) = $self->string_to_array($dimension);
     my $x_2 = $width/2;
     my $y_2 = $height/2;
     my $z_2 = defined $depth ? $depth/2 : $x_2;
     my $color_prop = "";
     if ($appearance) {
-	if ($appearance =~ /,/) {
-	    $color_prop = [0..4];
-	    $self->MaterialBinding("PER_FACE_INDEXED");
-	}
-	$self->Group->appearance($appearance);
+        if ($appearance =~ /,/) {
+            $color_prop = [0..4];
+            $self->MaterialBinding("PER_FACE_INDEXED");
+        }
+        $self->Group->appearance($appearance);
     }
     $self->Coordinate3("-$x_2 -$y_2 $z_2","$x_2 -$y_2 $z_2","$x_2 -$y_2 -$z_2","-$x_2 -$y_2 -$z_2","0 $y_2 0")
     ->IndexedFaceSet(["0, 1, 4","1, 2, 4","2, 3, 4","3, 0, 4","0, 3, 2, 1"],$color_prop);
@@ -514,21 +518,21 @@ sub text {
     $string =~ s/"/$quote/g if defined $string;
     $self->Group->appearance($appearance) if $appearance || $font;
     if (defined $string) {
-	if (ref($string)) {
-	    $string = '["'.join('","',@$string).'"]';
-	} else {
-	    $string =~ s/"/$quote/g;
-	    $string = "\"$string\"";
-	}
+        if (ref($string)) {
+            $string = '["'.join('","',@$string).'"]';
+        } else {
+            $string =~ s/"/$quote/g;
+            $string = "\"$string\"";
+        }
     }
     if (defined $font) {
-	my ($size, $family, $style) = split(/\s+/,$font,3);
+        my ($size, $family, $style) = split(/\s+/,$font,3);
         $self->FontStyle($size, $family, $style);
     }
     if (defined $align) {
-	$align =~ s/BEGIN/LEFT/i;
-	$align =~ s/MIDDLE/CENTER/i;
-	$align =~ s/END/RIGHT/i;
+        $align =~ s/BEGIN/LEFT/i;
+        $align =~ s/MIDDLE/CENTER/i;
+        $align =~ s/END/RIGHT/i;
     }
     $self->AsciiText($string, undef, $align);
     $self->End if $appearance || $font;
@@ -538,7 +542,7 @@ sub text {
 sub billtext {
     my $self = shift;
     $self->Separator;
-    $self->VRML_row("AxisAlignment { fields [SFBitMask alignment] alignment ALIGNAXISXYZ }\n") if $self->{'BROWSER'} =~ /$supported{'L3D_ext'}/i;
+    $self->_row("AxisAlignment { fields [SFBitMask alignment] alignment ALIGNAXISXYZ }\n") if $self->{'BROWSER'} =~ /$supported{'L3D_ext'}/i;
     $self->text(@_);
     $self->End;
 }
@@ -552,49 +556,49 @@ sub appearance {
         $texture, %material, $def, $defmat, $deftex);
     ITEM:
     foreach $item (split(/\s*;\s*/,$appearance_list)) {
-	($key,$value) = split(/\s*=\s*/,$item,2);
-	unless ($value) {	# color only
-	    $value = $key;
-	    $key = "diffuseColor";
-	}
-	MODE: {
-	    if ($key eq "d")  { $key = "diffuseColor";  last MODE; }
-	    if ($key eq "e")  { $key = "emissiveColor"; last MODE; }
-	    if ($key eq "s")  { $key = "specularColor"; last MODE; }
-	    if ($key eq "a")  { $key = "ambientColor";  last MODE; }
-	    if ($key eq "sh") { $key = "shininess";     last MODE; }
-	    if ($key eq "tr") { $key = "transparency";  last MODE; }
-	    if ($key eq "tex") { $texture = $value; next ITEM; }
-	    if ($key eq "def") { $def = $value; next ITEM; }
-	    if ($key eq "defmat") { $defmat = $value; next ITEM; }
-	    if ($key eq "deftex") { $deftex = $value; next ITEM; }
-	}
-	if ($key eq "diffuseColor" | $key eq "emissiveColor" | $key eq "specularColor" | $key eq "ambientColor") {
-	    if ($value =~ /,/) {	# multi color field
-		foreach $color (split(/\s*,\s*/,$value)) {
-		    ($num_color,$color) = rgb_color($color);
-		    $value = $num_color;
-		    $value .= "	# $color" if $color && $self->{'DEBUG'};
-		    push @values, $value;
-		}
-		$material{$key} = [@values];
-		$multi_color = 1;
-	    } else {
-		($num_color,$color) = rgb_color($value);
-		$value = $num_color;
-		$value .= "	# $color" if $color && $self->{'DEBUG'};
-		$material{$key} = $value;
-	    }
-	} else {
-		$material{$key} = $value;
-	}
+        ($key,$value) = split(/\s*=\s*/,$item,2);
+        unless ($value) {       # color only
+            $value = $key;
+            $key = "diffuseColor";
+        }
+        MODE: {
+            if ($key eq "d")  { $key = "diffuseColor";  last MODE; }
+            if ($key eq "e")  { $key = "emissiveColor"; last MODE; }
+            if ($key eq "s")  { $key = "specularColor"; last MODE; }
+            if ($key eq "a")  { $key = "ambientColor";  last MODE; }
+            if ($key eq "sh") { $key = "shininess";     last MODE; }
+            if ($key eq "tr") { $key = "transparency";  last MODE; }
+            if ($key eq "tex") { $texture = $value; next ITEM; }
+            if ($key eq "def") { $def = $value; next ITEM; }
+            if ($key eq "defmat") { $defmat = $value; next ITEM; }
+            if ($key eq "deftex") { $deftex = $value; next ITEM; }
+        }
+        if ($key eq "diffuseColor" | $key eq "emissiveColor" | $key eq "specularColor" | $key eq "ambientColor") {
+            if ($value =~ /,/) {        # multi color field
+                foreach $color (split(/\s*,\s*/,$value)) {
+                    ($num_color,$color) = rgb_color($color);
+                    $value = $num_color;
+                    $value .= " # $color" if $color && $self->{'DEBUG'};
+                    push @values, $value;
+                }
+                $material{$key} = [@values];
+                $multi_color = 1;
+            } else {
+                ($num_color,$color) = rgb_color($value);
+                $value = $num_color;
+                $value .= "     # $color" if $color && $self->{'DEBUG'};
+                $material{$key} = $value;
+            }
+        } else {
+                $material{$key} = $value;
+        }
     }
     $self->def($def)->group_begin if $def;
     $self->def($defmat) if $defmat;
     $self->Material(%material);
 #    $self->MaterialBinding("PER_FACE_INDEXED") if $multi_color;
     $self->def($deftex) if $deftex;
-    $self->Texture2(split(/\s+/,$texture)) if defined $texture;
+    $self->Texture2($self->string_to_array($texture)) if defined $texture;
     $self->group_end if $def;
     return $self;
 }
@@ -608,39 +612,39 @@ sub transform_begin {
     my (@transform_list) = @_;
     my @transform;
     if (ref($transform_list[0])) {
-	@transform = @{$transform_list[0]};
+        @transform = @{$transform_list[0]};
     } else {
-	@transform = @transform_list;
+        @transform = @transform_list;
     }
     my ($item, $key, $value);
     my ($x,$y,$z,$angle,$t,$r,$s,$o,$c);
     foreach $item (@transform) {
-	($key,$value) = ref($item) ? @$item : split(/\s*=\s*/,$item);
-	unless ($value) {
-	    ($x,$y,$z) = split(/\s/,$key);
-	    $x=0 unless defined $x;
-	    $y=0 unless defined $y;
-	    $z=0 unless defined $z;
-	    $t = "$x $y $z";
-	}
-	MODE: {
-	    if ($key eq "t") { $t = $value; last MODE; }
-	    if ($key eq "r" || $key eq "rotation") { $r = $value; last MODE; }
-	    if ($key eq "c") { $c = $value; last MODE; }
-	    if ($key eq "s") { $s = $value; last MODE; }
-	    if ($key eq "so") { $o = $value; last MODE; }
-	}
-	if ($key eq "r" || $key eq "rotation") {
-	    ($x,$y,$z,$angle) = split(/\s/,$value);
-	    unless ($angle) { # if not komplet assume first param is angle
-		$angle=$x;
-		$x=0;
-		$y=0;
-		$z=1;
-	    }
-	    $angle *= $::pi/180 if $self->{'CONVERT'};
-	    $r = "$x $y $z $angle";
-	}
+        ($key,$value) = ref($item) ? @$item : split(/\s*=\s*/,$item);
+        unless ($value) {
+            ($x,$y,$z) = split(/\s/,$key);
+            $x=0 unless defined $x;
+            $y=0 unless defined $y;
+            $z=0 unless defined $z;
+            $t = "$x $y $z";
+        }
+        MODE: {
+            if ($key eq "t") { $t = $value; last MODE; }
+            if ($key eq "r" || $key eq "rotation") { $r = $value; last MODE; }
+            if ($key eq "c") { $c = $value; last MODE; }
+            if ($key eq "s") { $s = $value; last MODE; }
+            if ($key eq "so") { $o = $value; last MODE; }
+        }
+        if ($key eq "r" || $key eq "rotation") {
+            ($x,$y,$z,$angle) = split(/\s/,$value);
+            unless (defined $angle) { # if one param its the angle
+                $angle=$x;
+                $x=0;
+                $y=0;
+                $z=1;
+            }
+            $angle *= $PI/180 if $self->{'CONVERT'};
+            $r = "$x $y $z $angle";
+        }
     }
     $self->Separator->Transform($t,$r,$s,$o,$c);
     return $self;
@@ -656,7 +660,7 @@ sub transform_end {
 
 sub sound {
     my $self = shift;
-    return $self->VRML_put(qq{# CALL: ->sound("url", "description", ...)\n}) unless @_;
+    return $self->_put(qq{# CALL: ->sound("url", "description", ...)\n}) unless @_;
     my ($url, $description, $location, $direction, $intensity, $loop, $pitch, $pause) = @_;
     $loop = defined $loop && $loop ? "TRUE" : "FALSE";
     $self->DirectedSound($url, $description, $location, $direction, $intensity, 100, 0, 0, 0, $loop);
@@ -671,22 +675,22 @@ sub def {
     $name = "_DEF_".(++$self->{'ID'}) unless defined $name;
     $self->DEF($name);
     if (defined $code) {
-	if (ref($code) eq "CODE") {
-	    $self->{'TAB'} .= "\t";
-	    my $pos = $#{$self->{'VRML'}}+1;
-	    &$code;
-	    $self->VRML_trim($pos);
-	    chop($self->{'TAB'});
-	} else {
-	    $self->VRML_put($code);
-	}
+        if (ref($code) eq "CODE") {
+            $self->{'TAB'} .= "\t";
+            my $pos = $#{$self->{'VRML'}}+1;
+            &$code;
+            $self->_trim($pos);
+            chop($self->{'TAB'});
+        } else {
+            $self->_put($code);
+        }
     }
     return $self;
 }
 
 sub use {
     my $self = shift;
-    return $self->VRML_put(qq{# CALL: ->use("name");\n}) unless @_;
+    return $self->_put(qq{# CALL: ->use("name");\n}) unless @_;
     my ($name) = @_;
     $self->USE($name);
     return $self;
@@ -696,10 +700,10 @@ sub AUTOLOAD {
     my $self = shift;
     $AUTOLOAD =~ s/.*:://g;
     unless ($AUTOLOAD =~ /^route|sensor$|^interpolator$|^elevationgrid$|^indexedfaceset$/) {
-	my ($package, $filename, $line)  = caller;
-	die qq{Unknown method "$AUTOLOAD" at $filename line $line.\n};
+        my ($package, $filename, $line)  = caller;
+        die qq{Unknown method "$AUTOLOAD" at $filename line $line.\n};
     }
-    return $self->VRML_row(qq{### "$AUTOLOAD" is not supported by VRML::VRML1\n});
+    return $self->_row(qq{### "$AUTOLOAD" is not supported by VRML::VRML1\n});
 }
 
 1;
